#pragma once

#ifndef POLYNOMIAL_HPP
#define POLYNOMIAL_HPP

#include <vector>
#include <initializer_list>
#include <ostream>

#include "vec2.hpp"

/** Model a polynomial
 *
 * Store the coefficient a[k] in the formula
 *  P(t)=Sum_k a[k] t^k
 *
 */
class polynomial
{
public:

    polynomial();
    polynomial(std::initializer_list<vec2> const& param);

    /** Get the degree of the polynomial (degree=nbr of coefficient -1)*/
    int degree() const;


    /** evaluate the polynomial P(t) at the given t */
    vec2 operator ()(float t) const;

    /** add a coefficient to the polynomial (increasing power of t) */
    void push_back(vec2 const& coefficient);

    /** stl compatible iterator */
    std::vector<vec2>::const_iterator begin() const;
    /** stl compatible iterator */
    std::vector<vec2>::const_iterator end() const;

private:
    /** internal data of coefficient */
    std::vector<vec2> data;
};

/** Print the polynomial */
std::ostream& operator<<(std::ostream& stream,polynomial const& param);

#endif
